# START_VERSION_BLOCK
VERSION_MAJOR = 0
VERSION_MINOR = 5
VERSION_BUILD = 2
VERSION_ALPHA = 1
# END_VERSION_BLOCK

VERSION_STR = f"{VERSION_MAJOR}.{VERSION_MINOR}.{VERSION_BUILD}"
if VERSION_ALPHA:
    VERSION_STR += f"a{VERSION_ALPHA}"
